<?xml version="1.0" encoding="UTF-8"?>
<!--This stylesheet will process a source XML file (ishare_audit.xml) and produce a summary of scope, including for example layers, workflow, for all the iShare instances listed in the source XML-->
<!--It will also if required copy a series of config files to a new location to support a migration of iShare-->
<!--This includes worfklow, datashare, mapsources and .map files-->
<!--To set the instances to be audited/migrated, edit the XML file, then run ishare_audit.bat to generate HTML/CSV output and if required copy the files-->
<!--This runs two XSLT processes - the first (this one) produces a detailed HTML report, CSV output and copies files, the second queries the full HTML to provide a summary HTML page-->
<!--Antony Scott, Astun Technology Oct 2016-->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:csv="csv:csv"
    xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xsl:output method="xml" encoding="utf-8"/>
    <xsl:strip-space elements="*"/>

    <!--HTML template-->
    <xsl:template match="/">
        <html>
            <head>
                <link rel="stylesheet" type="text/css" href="ishare_audit.css"/>
            </head>
            <body>
                <h1>Astun iShare Audit, run <xsl:value-of select="current-date()"/>, <xsl:value-of
                        select="current-time()"/></h1>
                <p>This report details the current scope of the Astun iShare installation, based on
                    the settings in Astun/Tools/Audit/config/ishare_audit.xml.</p>
                <p>A summary of this report is available at <a href="ishare_audit_summary.html"
                        >ishare_audit_summary.html</a>.</p>
                <!--Process each iShare instance-->
                <xsl:apply-templates select="audit-params/iShare"/>

                <!--Lists scheduled tasks if list has been generated-->
                <h3>Scheduled Tasks</h3>
                <xsl:variable name="v_tasks" select="'../output/tasks.xml'"/>
                <xsl:choose>
                    <xsl:when test="document($v_tasks)">
                        <xsl:call-template name="t_tasks">
                            <xsl:with-param name="p_tasks" select="$v_tasks"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </body>
        </html>
    </xsl:template>

    <!--This template will run for each instance of iShare specified in ishare_audit.xml-->
    <xsl:template match="iShare">
        <!--Set variables for this iShare instance-->
        <xsl:variable name="v_type" select="@type"/>
        <xsl:variable name="v_version" select="Version"/>
        <xsl:variable name="v_versiondest" select="VersionDest"/>
        <xsl:variable name="v_isharedataloc" select="iShareDataLoc"/>

        <!--Location of isharemaps.xml-->
        <xsl:variable name="v_isharemaps"
            select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/iShareMaps.xml'))"/>

        <!--Location of settings.xml-->
        <xsl:variable name="v_settings"
            select="string(concat('../../../',$v_type,'/',$v_version,'/Settings.xml'))"/>
        <xsl:variable name="v_data" select="document($v_settings)/Settings/data/config/@path"/>
        <xsl:variable name="v_mapfiles"
            select="string(concat(document($v_settings)/Settings/data/config/@name,'_MapServerConfig'))"/>

        <!--Used to find location of My Nearest - is there a better way?-->
        <xsl:variable name="v_data_config"
            select="substring-before(document($v_isharemaps)/MapSpur/DefaultMapSource,'/')"/>

        <!--Used to find location of My House - is there a better way?-->
        <xsl:variable name="v_myhouse"
            select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',document($v_isharemaps)/MapSpur/DefaultMyHouse[1],'.xml'))"/>

        <!--Location of Workflow.xml-->
        <xsl:variable name="v_workflow"
            select="string(concat('../../../',$v_type,'/',$v_version,'/Studio/config/Workflow.xml'))"/>

        <!--Location of iShareDataRoot.xml-->
        <xsl:variable name="v_isharedataroot"
            select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/iShareDataRoot.xml'))"/>

        <!--Location of datashare-->
        <xsl:variable name="v_datashare"
            select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',document($v_isharedataroot)/Configurations/Config,'.xml'))"/>

        <!--Location of the audit scripts root, for use with document output-->
        <xsl:variable name="v_base-folder"
            select="substring-after(substring-before(base-uri(),'config/ishare_audit.xml'),'file:/')"/>

        <!--Output files for migration if flag set in config file-->
        <xsl:if test="Migrate='true'">

            <!--Output Workflow-->
            <xsl:result-document method="xml" indent="yes"
                href="file:///{$v_base-folder}migrate/{$v_type}/{$v_versiondest}/Studio/config/Workflow.xml">
                <xsl:for-each select="document($v_workflow)/*">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="identity"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:result-document>

            <!--Output ErrorConfig-->
            <xsl:result-document method="xml" indent="yes"
                href="file:///{$v_base-folder}migrate/{$v_type}/{$v_versiondest}/Studio/config/ErrorConfig.xml">
                <xsl:for-each
                    select="document(string(concat('../../../',$v_type,'/',$v_version,'/Studio/config/ErrorConfig.xml')))/*">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="identity"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:result-document>

            <!--Output ConTypes-->
            <xsl:result-document method="xml" indent="yes"
                href="file:///{$v_base-folder}migrate/{$v_type}/{$v_versiondest}/Studio/config/ConTypes.xml">
                <xsl:for-each
                    select="document(string(concat('../../../',$v_type,'/',$v_version,'/Studio/config/ConTypes.xml')))/*">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="identity"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:result-document>

            <!--Output iShare Maps-->
            <xsl:result-document method="xml" indent="yes"
                href="file:///{$v_base-folder}migrate/{$v_type}/{$v_versiondest}/WebApps/WebService/config/iShareMaps.xml">
                <xsl:for-each select="document($v_isharemaps)/*">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="identity"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:result-document>

            <!--Output Datashare-->
            <xsl:result-document method="xml" indent="yes"
                href="file:///{$v_base-folder}migrate/{$v_type}/{$v_versiondest}/WebApps/WebService/config/{document($v_isharedataroot)/Configurations/Config}.xml">
                <xsl:for-each select="document($v_datashare)/*">
                    <xsl:copy>
                        <xsl:apply-templates select="@*|node()" mode="identity"/>
                    </xsl:copy>
                </xsl:for-each>
            </xsl:result-document>

            <!--Output MapSources-->
            <!--Standard Map Sources-->
            <xsl:for-each select="document($v_isharemaps)/MapSpur/MapSources/MapSource">
                <xsl:call-template name="t_mapsource">
                    <xsl:with-param name="p_mapname" select="MapName"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_versiondest" select="$v_versiondest"/>
                    <xsl:with-param name="p_base-folder" select="$v_base-folder"/>
                    <xsl:with-param name="p_isharedataloc" select="$v_isharedataloc"/>
                </xsl:call-template>
            </xsl:for-each>

            <!--Base Map Sources-->
            <xsl:for-each select="document($v_isharemaps)/MapSpur/BaseMapSources/BaseMapSource">
                <xsl:call-template name="t_mapsource">
                    <xsl:with-param name="p_mapname" select="MapName"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_versiondest" select="$v_versiondest"/>
                    <xsl:with-param name="p_base-folder" select="$v_base-folder"/>
                    <xsl:with-param name="p_isharedataloc" select="$v_isharedataloc"/>
                </xsl:call-template>
            </xsl:for-each>
            <xsl:if test="$v_type='iShare'">

                <!--My House-->
                <xsl:call-template name="t_mapsource">
                    <xsl:with-param name="p_mapname"
                        select="document($v_isharemaps)/MapSpur/document($v_isharemaps)/MapSpur/DefaultMyHouse[1]/text()"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_versiondest" select="$v_versiondest"/>
                    <xsl:with-param name="p_base-folder" select="$v_base-folder"/>
                    <xsl:with-param name="p_isharedataloc" select="$v_isharedataloc"/>
                </xsl:call-template>

                <!--My Nearest if exists-->
                <xsl:if
                    test="doc-available(string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',$v_data_config,'/MyNearest.xml')))">
                    <xsl:call-template name="t_mapsource">
                        <xsl:with-param name="p_mapname"
                            select="string(concat($v_data_config,'/MyNearest'))"/>
                        <xsl:with-param name="p_type" select="$v_type"/>
                        <xsl:with-param name="p_version" select="$v_version"/>
                        <xsl:with-param name="p_versiondest" select="$v_versiondest"/>
                        <xsl:with-param name="p_base-folder" select="$v_base-folder"/>
                        <xsl:with-param name="p_isharedataloc" select="$v_isharedataloc"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:if>
        </xsl:if>
        <!--End of migration section-->

        <!--Output CSVs-->
        <!--Output DataShare list to csv-->
        <xsl:result-document method="text"
            href="file:///{$v_base-folder}output/{$v_type}_{$v_version}_datashare.csv">
            <xsl:text>Display Name, Group, DSN&#xA;</xsl:text>
            <xsl:for-each select="document($v_datashare)/iShareData/DataSources/DataSource">
                <xsl:value-of
                    select="concat('&quot;',DisplayName,'&quot;',',','&quot;',GroupName,'&quot;',',','&quot;',Type/DSN,'&quot;','&#xA;')"
                />
            </xsl:for-each>
        </xsl:result-document>

        <!--Output layer lists to csv-->
        <xsl:result-document method="text"
            href="file:///{$v_base-folder}output/{$v_type}_{$v_version}_layers.csv">
            <xsl:text>Map Source, Layer Group, Layer, MapServer Layer, Type, .map file&#xA;</xsl:text>
            <!--Map Sources-->
            <xsl:for-each select="document($v_isharemaps)/MapSpur/MapSources/MapSource">
                <xsl:call-template name="t_layer_list">
                    <xsl:with-param name="p_map_source"
                        select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',MapName,'.xml'))"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_mapname" select="MapName"/>
                </xsl:call-template>
            </xsl:for-each>
            <xsl:if test="$v_type='iShare'">
                <!--My Nearest-->
                <xsl:call-template name="t_layer_list">
                    <xsl:with-param name="p_map_source"
                        select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',$v_data_config,'/MyNearest.xml'))"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_mapname" select="'MyNearest'"/>
                </xsl:call-template>
                <!--My House-->
                <xsl:call-template name="t_layer_list">
                    <xsl:with-param name="p_map_source" select="$v_myhouse"/>
                    <xsl:with-param name="p_type" select="$v_type"/>
                    <xsl:with-param name="p_version" select="$v_version"/>
                    <xsl:with-param name="p_mapname" select="'MyHouse'"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:result-document>

        <!--Output task/job list-->
        <xsl:result-document method="text"
            href="file:///{$v_base-folder}output/{$v_type}_{$v_version}_workflow.csv">
            <xsl:call-template name="t_workflow_task_list">
                <xsl:with-param name="p_workflow" select="$v_workflow"/>
                <xsl:with-param name="p_type" select="$v_type"/>
                <xsl:with-param name="p_version" select="$v_version"/>
            </xsl:call-template>
        </xsl:result-document>
        <!--End output CSV-->

        <!--Output HTML-->
        <h2>
            <xsl:value-of select="$v_type"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="$v_version"/>
        </h2>

        <!--Iterate through Map Sources, count layers-->
        <h3>Map Sources</h3>
        <xsl:for-each select="document($v_isharemaps)/MapSpur/MapSources/MapSource">
            <h4 type="MapSource" instance="{concat($v_type,' ',$v_version)}">Name: <xsl:value-of
                    select="MapName"/></h4>
            <xsl:call-template name="t_map_sources">
                <xsl:with-param name="p_map_source"
                    select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',MapName,'.xml'))"/>
                <xsl:with-param name="p_type" select="$v_type"/>
                <xsl:with-param name="p_version" select="$v_version"/>
            </xsl:call-template>
        </xsl:for-each>

        <!--Count My Nearest layers-->
        <xsl:if test="$v_type='iShare'">
            <h3>My Nearest</h3>
            <xsl:call-template name="t_map_sources">
                <xsl:with-param name="p_map_source"
                    select="string(concat('../../../',$v_type,'/',$v_version,'/WebApps/WebService/config/',$v_data_config,'/MyNearest.xml'))"/>
                <xsl:with-param name="p_type" select="$v_type"/>
                <xsl:with-param name="p_version" select="$v_version"/>
            </xsl:call-template>

            <!--Count My House layers-->
            <h3>My House</h3>
            <xsl:call-template name="t_map_sources">
                <xsl:with-param name="p_map_source" select="$v_myhouse"/>
                <xsl:with-param name="p_type" select="$v_type"/>
                <xsl:with-param name="p_version" select="$v_version"/>
            </xsl:call-template>
        </xsl:if>

        <!--List Base layers-->
        <h3>Base Layers</h3>
        <xsl:call-template name="t_base_maps">
            <xsl:with-param name="p_isharemaps" select="$v_isharemaps"/>
            <xsl:with-param name="p_type" select="$v_type"/>
            <xsl:with-param name="p_version" select="$v_version"/>
        </xsl:call-template>

        <!--Count Workflow Jobs and Tasks-->
        <h3>Workflow</h3>
        <xsl:call-template name="t_workflow_jobs">
            <xsl:with-param name="p_workflow" select="$v_workflow"/>
            <xsl:with-param name="p_type" select="$v_type"/>
            <xsl:with-param name="p_version" select="$v_version"/>
        </xsl:call-template>

        <!--List datashares-->
        <h3>DataShare</h3>
        <xsl:call-template name="t_datashare">
            <xsl:with-param name="p_datashare" select="$v_datashare"/>
            <xsl:with-param name="p_type" select="$v_type"/>
            <xsl:with-param name="p_version" select="$v_version"/>
        </xsl:call-template>

        <!--List functions-->
        <h3>Functions Used (Gazetteer in bold)</h3>
        <xsl:call-template name="t_workflow_functs">
            <xsl:with-param name="p_workflow" select="$v_workflow"/>
            <xsl:with-param name="p_type" select="$v_type"/>
            <xsl:with-param name="p_version" select="$v_version"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="*"/>

    <!--Named templates-->
    <xsl:template name="t_layer_list">
        <xsl:param name="p_map_source"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <xsl:param name="p_mapname"/>
        <xsl:for-each select="document($p_map_source)/MapSource/Layers/LayerGroup/Layer">
            <!--Map Source, Layer Group, Layer, MapServer Layer, Type, .map file-->
            <xsl:value-of
                select="concat($p_mapname,',','&quot;',../Name,'&quot;',',','&quot;',Name,'&quot;',',',LayerName,',',Type,',',../../../MapName,'&#xA;')"
            />
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="t_map_sources">
        <xsl:param name="p_map_source"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <table>
            <tr>
                <th/>
                <th>Layer Group Name</th>
                <th>No. Layers</th>
            </tr>
            <xsl:for-each select="document($p_map_source)/MapSource/Layers/LayerGroup">
                <tr>
                    <td/>
                    <td>
                        <xsl:value-of select="Name"/>
                    </td>
                    <td>
                        <xsl:value-of select="count(Layer)"/>
                    </td>
                </tr>
            </xsl:for-each>
            <tr instance="{concat($p_type,' ',$p_version)}">
                <td>Totals:</td>
                <td type="LayerGroup">
                    <xsl:value-of
                        select="count(document($p_map_source)/MapSource/Layers/LayerGroup)"/>
                </td>
                <td type="Layer">
                    <xsl:value-of
                        select="count(document($p_map_source)/MapSource/Layers/LayerGroup/Layer)"/>
                </td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template name="t_datashare">
        <xsl:param name="p_datashare"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <table>
            <tr>
                <th/>
                <th>DataShare</th>
                <th>No.</th>
            </tr>
            <xsl:for-each select="document($p_datashare)/iShareData/DataSources">
                <tr instance="{concat($p_type,' ',$p_version)}">
                    <td>Total:</td>
                    <td/>
                    <td type="DataShare">
                        <xsl:value-of select="count(DataSource)"/>
                    </td>
                </tr>
            </xsl:for-each>
        </table>
    </xsl:template>

    <xsl:template name="t_base_maps">
        <xsl:param name="p_isharemaps"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <table>
            <tr>
                <th/>
                <th>Map Source Name</th>
                <th>Map Source Location</th>
            </tr>
            <xsl:for-each select="document($p_isharemaps)/MapSpur/BaseMapSources/BaseMapSource">
                <tr>
                    <td/>
                    <td>
                        <xsl:value-of select="DisplayName"/>
                    </td>
                    <td>
                        <xsl:value-of select="MapName"/>
                    </td>
                </tr>
            </xsl:for-each>
            <tr instance="{concat($p_type,' ',$p_version)}">
                <td>Totals:</td>
                <td type="BaseLayer">
                    <xsl:value-of
                        select="count(document($p_isharemaps)/MapSpur/BaseMapSources/BaseMapSource)"
                    />
                </td>
                <td> </td>
            </tr>
        </table>
    </xsl:template>
    <xsl:template name="t_workflow_jobs">
        <xsl:param name="p_workflow"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <table>
            <tr>
                <th/>
                <th>Jobs</th>
                <th>Tasks</th>
            </tr>
            <xsl:for-each select="document($p_workflow)/Workflow/Jobs/Job">
                <tr>
                    <td/>
                    <td>
                        <xsl:value-of select="Name"/>
                    </td>
                    <td>
                        <xsl:value-of select="count(Task)"/>
                    </td>
                </tr>
            </xsl:for-each>
            <tr instance="{concat($p_type,' ',$p_version)}">
                <td>Totals:</td>
                <td type="Job">
                    <xsl:value-of select="count(document($p_workflow)/Workflow/Jobs/Job)"/>
                </td>
                <td type="Task">
                    <xsl:value-of select="count(document($p_workflow)/Workflow/Tasks/Task)"/>
                </td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template name="t_workflow_task_list">
        <xsl:param name="p_workflow"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <xsl:text>Job Name,Task Name,Type,Source Connection,Source,Dest. Connection,Dest.,Connection,Stored Procedure&#xA;</xsl:text>
        <xsl:for-each select="document($p_workflow)/Workflow/Tasks/Task">
            <xsl:variable name="v_task_name" select="Name"/>
            <!--Where tasks appear in multiple jobs, only the first job is included-->
            <xsl:value-of
                select="concat('&quot;',../../Jobs/Job[Task/TaskName=$v_task_name][1]/Name,'&quot;',',','&quot;',Name,'&quot;',',',Type,',',SourceConnetion,',','&quot;',SourceValue,'&quot;',',',DestConnection,',','&quot;',DestValue,'&quot;',',',Connection,',',StoredProcedure,'&#xA;')"
            />
        </xsl:for-each>
        <xsl:for-each
            select="document($p_workflow)/Workflow/Jobs/Job/Task[contains(TaskName,'[DataShare]')]">
            <xsl:value-of
                select="concat('&quot;',../Name,'&quot;',',','&quot;',TaskName,'&quot;,','DataShare',',',SourceConnetion,',','&quot;',SourceValue,'&quot;',',',DestConnection,',','&quot;',DestValue,'&quot;',',',Connection,',',StoredProcedure,'&#xA;')"
            />
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="t_workflow_functs">
        <xsl:param name="p_workflow"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <table>
            <tr>
                <th/>
                <th>Task Name</th>
                <th>Function</th>
            </tr>
            <xsl:for-each
                select="document($p_workflow)/Workflow/Tasks/Task/StoredProcedure[starts-with(text(),'at_')]">
                <tr>
                    <td/>
                    <xsl:choose>
                        <xsl:when test="starts-with(text(),'at_wkf_build')">
                            <td>
                                <b>
                                    <xsl:value-of select="../Name"/>
                                </b>
                            </td>
                            <td>
                                <b>
                                    <xsl:value-of select="text()"/>
                                </b>
                            </td>
                        </xsl:when>
                        <xsl:otherwise>
                            <td>
                                <xsl:value-of select="../Name"/>
                            </td>
                            <td>
                                <xsl:value-of select="text()"/>
                            </td>
                        </xsl:otherwise>
                    </xsl:choose>
                </tr>
            </xsl:for-each>
            <tr instance="{concat($p_type,' ',$p_version)}">
                <td>Totals (Astun):</td>
                <td> </td>
                <td type="FunctionA">
                    <xsl:value-of
                        select="count(document($p_workflow)/Workflow/Tasks/Task/StoredProcedure[starts-with(text(),'at_')])"
                    />
                </td>
            </tr>
            <xsl:for-each
                select="document($p_workflow)/Workflow/Tasks/Task/StoredProcedure[not(starts-with(text(),'at_'))]">
                <tr>
                    <td/>
                    <td>
                        <xsl:value-of select="../Name"/>
                    </td>
                    <td>
                        <xsl:value-of select="text()"/>
                    </td>
                </tr>
            </xsl:for-each>
            <tr instance="{concat($p_type,' ',$p_version)}">
                <td>Totals (Non-Astun):</td>
                <td> </td>
                <td type="FunctionNA">
                    <xsl:value-of
                        select="count(document($p_workflow)/Workflow/Tasks/Task/StoredProcedure[not(starts-with(text(),'at_'))])"
                    />
                </td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template name="t_tasks">
        <xsl:param name="p_tasks"/>
        <table>
            <tr>
                <th>Task</th>
                <th>Frequency</th>
                <th>Time</th>
                <th>Day</th>
            </tr>
            <xsl:for-each
                select="document($p_tasks)/CSVData/Dataset/data[h8 = 'Astun Technology DataShare Synchroniser']">
                <tr>
                    <td>
                        <xsl:value-of select="h9"/>
                    </td>
                    <td>
                        <xsl:value-of select="h19"/>
                    </td>
                    <td>
                        <xsl:value-of select="h20"/>
                    </td>
                    <td>
                        <xsl:value-of select="h23"/>
                    </td>
                </tr>
            </xsl:for-each>
            <tr>
                <td>Totals:</td>
                <td>
                    <xsl:value-of
                        select="count(document($p_tasks)/CSVData/Dataset/data[h8 = 'Astun Technology DataShare Synchroniser'])"
                    />
                </td>
                <td> </td>
                <td> </td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template name="t_mapsource">
        <xsl:param name="p_mapname"/>
        <xsl:param name="p_type"/>
        <xsl:param name="p_version"/>
        <xsl:param name="p_versiondest"/>
        <xsl:param name="p_base-folder"/>
        <xsl:param name="p_isharedataloc"/>
        <xsl:result-document method="xml" indent="yes"
            href="file:///{$p_base-folder}migrate/{$p_type}/{$p_versiondest}/WebApps/WebService/config/{$p_mapname}.xml">
            <xsl:for-each
                select="document(string(concat('../../../',$p_type,'/',$p_version,'/WebApps/WebService/config/',$p_mapname,'.xml')))">
                <xsl:choose>
                    <xsl:when test="$p_isharedataloc">
                        <xsl:copy copy-namespaces="no">
                            <xsl:apply-templates select="@*|node()" mode="isharedata">
                                <xsl:with-param name="p_isharedataloc" select="$p_isharedataloc"/>
                            </xsl:apply-templates>
                        </xsl:copy>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:copy>
                            <xsl:apply-templates select="@*|node()" mode="identity"/>
                        </xsl:copy>
                    </xsl:otherwise>
                </xsl:choose>
                <!--Output .map file for each Map Source to isharedata-->
                <!--Set destination filename-->
                <xsl:variable name="v_path_out">
                    <xsl:value-of
                        select="concat('file:///',$p_base-folder,'migrate/isharedata',translate(substring-after(MapSource/MapName,'iShareData'),'\','/'))"
                    />
                </xsl:variable>
                <xsl:variable name="v_path_in">
                    <xsl:value-of select="concat('file:///',translate(MapSource/MapName,'\','/'))"/>
                </xsl:variable>
                <xsl:choose>
                    <!--Check if file already exists to avoid XSLT error if it does-->
                    <xsl:when test="unparsed-text-available($v_path_out)"/>
                    <xsl:otherwise>
                        <xsl:result-document method="text" href="{$v_path_out}">
                            <xsl:copy-of select="unparsed-text($v_path_in)"/>
                        </xsl:result-document>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:result-document>
    </xsl:template>
    <!--End named templates-->

    <xsl:template match="@*|node()" mode="identity">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" mode="identity"/>
        </xsl:copy>
    </xsl:template>

    <!--Replace drive letter in MapName-->
    <xsl:template match="*/MapName" mode="isharedata">
        <xsl:param name="p_isharedataloc"/>
        <xsl:copy copy-namespaces="no">
            <xsl:value-of select="concat($p_isharedataloc,':',substring-after(text(),':'))"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="@*|node()" mode="isharedata">
        <xsl:param name="p_isharedataloc"/>
        <xsl:copy copy-namespaces="no">
            <xsl:apply-templates select="@*|node()" mode="isharedata">
                <xsl:with-param name="p_isharedataloc" select="$p_isharedataloc"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
